import React, { useState, useEffect } from 'react';
import { Box, Typography, TextField, Button, Select, MenuItem, Snackbar } from '@mui/material';
import QRCode from 'react-qr-code';
import { ContactPerson, Address, Dewar, ContactsService, AddressesService, ShipmentsService, Puck, Sample } from '../../openapi';
import Unipuck from '../components/Unipuck';

interface DewarDetailsProps {
    dewar: Dewar;
    trackingNumber: string;
    setTrackingNumber: React.Dispatch<React.SetStateAction<string>>;
    initialContactPersons: ContactPerson[];
    initialReturnAddresses: Address[];
    defaultContactPerson?: ContactPerson;
    defaultReturnAddress?: Address;
    shipmentId: string;
    refreshShipments: () => void;
    selectedShipment: any;
}

const DewarDetails: React.FC<DewarDetailsProps> = ({
                                                       dewar,
                                                       trackingNumber,
                                                       setTrackingNumber,
                                                       initialContactPersons = [],
                                                       initialReturnAddresses = [],
                                                       defaultContactPerson,
                                                       defaultReturnAddress,
                                                       shipmentId,
                                                       refreshShipments,
                                                       selectedShipment
                                                   }) => {
    const [localTrackingNumber, setLocalTrackingNumber] = useState(trackingNumber);
    const [contactPersons, setContactPersons] = useState<ContactPerson[]>(initialContactPersons);
    const [returnAddresses, setReturnAddresses] = useState<Address[]>(initialReturnAddresses);
    const [selectedContactPerson, setSelectedContactPerson] = useState<string>('');
    const [selectedReturnAddress, setSelectedReturnAddress] = useState<string>('');
    const [isCreatingContactPerson, setIsCreatingContactPerson] = useState(false);
    const [isCreatingReturnAddress, setIsCreatingReturnAddress] = useState(false);
    const [puckStatuses, setPuckStatuses] = useState<string[][]>(dewar.pucks.map(() => Array(16).fill('empty')));
    const [newContactPerson, setNewContactPerson] = useState({
        id: 0,
        firstName: '',
        lastName: '',
        phone_number: '',
        email: '',
    });
    const [newReturnAddress, setNewReturnAddress] = useState<Address>({
        id: 0,
        street: '',
        city: '',
        zipcode: '',
        country: '',
    });
    const [changesMade, setChangesMade] = useState<boolean>(false);
    const [feedbackMessage, setFeedbackMessage] = useState<string>('');
    const [openSnackbar, setOpenSnackbar] = useState<boolean>(false);

    useEffect(() => {
        const setInitialContactPerson = () => {
            const contactPersonId =
                selectedShipment?.contact_person?.id?.toString() ||
                dewar.contact_person?.id?.toString() ||
                defaultContactPerson?.id?.toString() ||
                '';
            setSelectedContactPerson(contactPersonId);
        };

        const setInitialReturnAddress = () => {
            const returnAddressId =
                dewar.return_address?.id?.toString() ||
                defaultReturnAddress?.id?.toString() ||
                '';
            setSelectedReturnAddress(returnAddressId);
        };

        setLocalTrackingNumber(dewar.tracking_number || '');
        setInitialContactPerson();
        setInitialReturnAddress();
    }, [dewar, defaultContactPerson, defaultReturnAddress, selectedShipment]);

    useEffect(() => {
        const getContacts = async () => {
            try {
                const c: ContactPerson[] = await ContactsService.getContactsContactsGet();
                setContactPersons(c);
            } catch {
                setFeedbackMessage('Failed to load contact persons. Please try again later.');
                setOpenSnackbar(true);
            }
        };

        const getReturnAddresses = async () => {
            try {
                const a: Address[] = await AddressesService.getReturnAddressesAddressesGet();
                setReturnAddresses(a);
            } catch {
                setFeedbackMessage('Failed to load return addresses. Please try again later.');
                setOpenSnackbar(true);
            }
        };

        getContacts();
        getReturnAddresses();
    }, []);

    useEffect(() => {
        const fetchSamples = async () => {
            if (dewar.id) {
                try {
                    const samples: Sample[] = await ShipmentsService.getSamplesInDewarShipmentsShipmentIdDewarsDewarIdSamplesGet(shipmentId, dewar.id);
                    const updatedPuckStatuses = dewar.pucks.map(puck => {
                        return puck.positions.map(position => {
                            const isOccupied = samples.some(sample => sample.id === position.id);
                            return isOccupied ? 'filled' : 'empty';
                        });
                    });
                    setPuckStatuses(updatedPuckStatuses);
                } catch {
                    setFeedbackMessage('Failed to load samples. Please try again later.');
                    setOpenSnackbar(true);
                }
            }
        };

        fetchSamples();
    }, [dewar, shipmentId]);

    const validateEmail = (email: string) => /\S+@\S+\.\S+/.test(email);
    const validatePhoneNumber = (phone: string) => /^\+?[1-9]\d{1,14}$/.test(phone);
    const validateZipCode = (zipcode: string) => /^\d{5}(?:[-\s]\d{4})?$/.test(zipcode);

    if (!dewar) {
        return <Typography>No dewar selected.</Typography>;
    }

    const handleAddContact = async () => {
        if (!validateEmail(newContactPerson.email) || !validatePhoneNumber(newContactPerson.phone_number) ||
            !newContactPerson.firstName || !newContactPerson.lastName) {
            setFeedbackMessage('Please fill in all new contact person fields correctly.');
            setOpenSnackbar(true);
            return;
        }

        const payload = {
            firstname: newContactPerson.firstName,
            lastname: newContactPerson.lastName,
            phone_number: newContactPerson.phone_number,
            email: newContactPerson.email,
        };

        try {
            const c: ContactPerson = await ContactsService.createContactContactsPost(payload);
            setContactPersons([...contactPersons, c]);
            setFeedbackMessage('Contact person added successfully.');
            setNewContactPerson({ id: 0, firstName: '', lastName: '', phone_number: '', email: '' });
            setSelectedContactPerson(c.id?.toString() || '');
        } catch {
            setFeedbackMessage('Failed to create a new contact person. Please try again later.');
        }

        setOpenSnackbar(true);
        setIsCreatingContactPerson(false);
        setChangesMade(true);
    };

    const handleAddAddress = async () => {
        if (!validateZipCode(newReturnAddress.zipcode) || !newReturnAddress.street || !newReturnAddress.city || !newReturnAddress.country) {
            setFeedbackMessage('Please fill in all new return address fields correctly.');
            setOpenSnackbar(true);
            return;
        }

        const payload = {
            street: newReturnAddress.street.trim(),
            city: newReturnAddress.city.trim(),
            zipcode: newReturnAddress.zipcode.trim(),
            country: newReturnAddress.country.trim(),
        };

        try {
            const a: Address = await AddressesService.createReturnAddressAddressesPost(payload);
            setReturnAddresses([...returnAddresses, a]);
            setFeedbackMessage('Return address added successfully.');
            setNewReturnAddress({ id: 0, street: '', city: '', zipcode: '', country: '' });
            setSelectedReturnAddress(a.id?.toString() || '');
        } catch {
            setFeedbackMessage('Failed to create a new return address. Please try again later.');
        }

        setOpenSnackbar(true);
        setIsCreatingReturnAddress(false);
        setChangesMade(true);
    };

    const getShipmentById = async (shipmentId: string) => {
        try {
            const response = await ShipmentsService.fetchShipmentsShipmentsGet(shipmentId);
            if (response && response.length > 0) {
                return response[0];
            }
            throw new Error('Shipment not found');
        } catch (error) {
            throw error;
        }
    };

    const handleSaveChanges = async () => {
        const formatDate = (dateString: string | undefined): string | null => {
            if (!dateString) return null;
            const date = new Date(dateString);
            if (isNaN(date.getTime())) return null;
            return date.toISOString().split('T')[0];
        };

        if (!selectedContactPerson || !selectedReturnAddress) {
            setFeedbackMessage('Please ensure all required fields are filled.');
            setOpenSnackbar(true);
            return;
        }

        let existingShipment;
        try {
            existingShipment = await getShipmentById(shipmentId);
        } catch {
            setFeedbackMessage('Failed to fetch existing shipment data. Please try again later.');
            setOpenSnackbar(true);
            return;
        }

        const updatedDewar = {
            dewar_id: dewar.id,
            dewar_name: dewar.dewar_name,
            tracking_number: localTrackingNumber,
            number_of_pucks: dewar.number_of_pucks,
            number_of_samples: dewar.number_of_samples,
            status: dewar.status,
            ready_date: formatDate(dewar.ready_date ?? undefined),
            shipping_date: formatDate(dewar.shipping_date ?? undefined),
            arrival_date: dewar.arrival_date,
            returning_date: dewar.returning_date,
            qrcode: dewar.qrcode,
            return_address_id: selectedReturnAddress,
            contact_person_id: selectedContactPerson,
        };

        const payload = {
            shipment_id: existingShipment.shipment_id,
            shipment_name: existingShipment.shipment_name,
            shipment_date: existingShipment.shipment_date,
            shipment_status: existingShipment.shipment_status,
            comments: existingShipment.comments,
            contact_person_id: existingShipment.contact_person.id,
            return_address_id: selectedReturnAddress,
            proposal_id: existingShipment.proposal?.id,
            dewars: [updatedDewar],
        };

        try {
            await ShipmentsService.updateShipmentShipmentsShipmentIdPut(shipmentId, payload);
            setFeedbackMessage('Changes saved successfully.');
            setChangesMade(false);
            refreshShipments();
        } catch (error: any) {
            if (error.response && error.response.data) {
                setFeedbackMessage(`Failed to save shipment. Validation errors: ${JSON.stringify(error.response.data)}`);
            } else {
                setFeedbackMessage('Failed to save changes. Please try again later.');
            }
            setOpenSnackbar(true);
        }
    };

    return (
        <Box sx={{ marginTop: 2 }}>
            <Typography variant="h6">Selected Dewar: {dewar.dewar_name}</Typography>
            <TextField
                label="Tracking Number"
                value={localTrackingNumber}
                onChange={(e) => {
                    setLocalTrackingNumber(e.target.value);
                    setTrackingNumber(e.target.value);
                    setChangesMade(true);
                }}
                variant="outlined"
                sx={{ width: '300px', marginBottom: 2 }}
            />
            <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                <Box sx={{ display: 'flex', alignItems: 'center', justifyContent: 'center', flexDirection: 'column' }}>
                    {dewar.qrcode ? (
                        <QRCode value={dewar.qrcode} size={70} />
                    ) : (
                        <Typography>No QR code available</Typography>
                    )}
                    <Button variant="contained" sx={{ marginTop: 1 }} onClick={() => { /* Add logic to generate QR Code */ }}>
                        Generate QR Code
                    </Button>
                </Box>
            </Box>
            <Typography variant="body1">Number of Pucks: {dewar.number_of_pucks}</Typography>
            <Box sx={{ marginTop: 2 }}>
                {/* Other inputs and elements */}

                <Typography variant="body1">Number of Pucks: {dewar.number_of_pucks}</Typography>

                {/* Here we integrate the Unipuck component with puck data */}
                {puckStatuses && <Unipuck pucks={puckStatuses.length} samples={puckStatuses} />}

                <Typography variant="body1">Number of Samples: {dewar.number_of_samples}</Typography>
                {/* Rest of DewarDetails component */}
            </Box>
            <Typography variant="body1">Number of Samples: {dewar.number_of_samples}</Typography>
            <Typography variant="body1">Current Contact Person:</Typography>
            <Select
                value={selectedContactPerson}
                onChange={(e) => {
                    const value = e.target.value;
                    setSelectedContactPerson(value);
                    setIsCreatingContactPerson(value === 'add');
                    setChangesMade(true);
                }}
                fullWidth
                sx={{ marginBottom: 2 }}
                variant="outlined"
                displayEmpty
            >
                {Array.isArray(contactPersons) && contactPersons.map((person) => (
                    <MenuItem key={person.id?.toString()} value={person.id?.toString() || ''}>
                        {person.firstname} {person.lastname}
                    </MenuItem>
                ))}
                <MenuItem value="add">Add New Contact Person</MenuItem>
            </Select>
            {isCreatingContactPerson && (
                <Box sx={{ marginBottom: 2 }}>
                    <TextField
                        label="First Name"
                        value={newContactPerson.firstName}
                        onChange={(e) => setNewContactPerson({ ...newContactPerson, firstName: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                    />
                    <TextField
                        label="Last Name"
                        value={newContactPerson.lastName}
                        onChange={(e) => setNewContactPerson({ ...newContactPerson, lastName: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                    />
                    <TextField
                        label="Phone"
                        value={newContactPerson.phone_number}
                        onChange={(e) => setNewContactPerson({ ...newContactPerson, phone_number: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                        error={!validatePhoneNumber(newContactPerson.phone_number)}
                        helperText={!validatePhoneNumber(newContactPerson.phone_number) ? "Invalid phone number" : ""}
                    />
                    <TextField
                        label="Email"
                        value={newContactPerson.email}
                        onChange={(e) => setNewContactPerson({ ...newContactPerson, email: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                        error={!validateEmail(newContactPerson.email)}
                        helperText={!validateEmail(newContactPerson.email) ? "Invalid email" : ""}
                    />
                    <Button variant="contained" onClick={handleAddContact}>
                        Save Contact Person
                    </Button>
                </Box>
            )}
            <Typography variant="body1">Current Return Address:</Typography>
            <Select
                value={selectedReturnAddress}
                onChange={(e) => {
                    const value = e.target.value;
                    setSelectedReturnAddress(value);
                    setIsCreatingReturnAddress(value === 'add');
                    setChangesMade(true);
                }}
                fullWidth
                sx={{ marginBottom: 2 }}
                variant="outlined"
                displayEmpty
            >
                {Array.isArray(returnAddresses) && returnAddresses.map((address) => (
                    <MenuItem key={address.id?.toString()} value={address.id?.toString() || ''}>
                        {address.street}, {address.city}
                    </MenuItem>
                ))}
                <MenuItem value="add">Add New Return Address</MenuItem>
            </Select>
            {isCreatingReturnAddress && (
                <Box sx={{ marginBottom: 2 }}>
                    <TextField
                        label="Street"
                        value={newReturnAddress.street}
                        onChange={(e) => setNewReturnAddress({ ...newReturnAddress, street: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                    />
                    <TextField
                        label="City"
                        value={newReturnAddress.city}
                        onChange={(e) => setNewReturnAddress({ ...newReturnAddress, city: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                    />
                    <TextField
                        label="Zip Code"
                        value={newReturnAddress.zipcode}
                        onChange={(e) => setNewReturnAddress({ ...newReturnAddress, zipcode: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                        error={!validateZipCode(newReturnAddress.zipcode)}
                        helperText={!validateZipCode(newReturnAddress.zipcode) ? "Invalid zip code" : ""}
                    />
                    <TextField
                        label="Country"
                        value={newReturnAddress.country}
                        onChange={(e) => setNewReturnAddress({ ...newReturnAddress, country: e.target.value })}
                        variant="outlined"
                        fullWidth
                        sx={{ marginBottom: 1 }}
                    />
                    <Button variant="contained" onClick={handleAddAddress}>
                        Save Return Address
                    </Button>
                </Box>
            )}
            {changesMade && (
                <Button variant="contained" color="primary" onClick={handleSaveChanges}>
                    Save Changes
                </Button>
            )}
            <Snackbar
                open={openSnackbar}
                autoHideDuration={6000}
                onClose={() => setOpenSnackbar(false)}
                message={feedbackMessage}
            />
        </Box>
    );
};

export default DewarDetails;